/*
 * 2D Game Engine 
 * CameraMovement.h: 
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All rights reserved.
 *
 * Created by Manoël Trapier on 26/02/2021.
 */

#ifndef GAMEENGINE_SOURCE_INCLUDE_SYSTEMS_CAMERAMOVEMENT_H
#define GAMEENGINE_SOURCE_INCLUDE_SYSTEMS_CAMERAMOVEMENT_H

#include <SDL.h>

#include <ECS.h>

#include <Components/CameraFollow.h>
#include <Components/Transform.h>

class CameraMovementSystem: public System
{
public:
    CameraMovementSystem()
    {
        this->requireComponent<CameraFollowComponent>();
        this->requireComponent<TransformComponent>();
    }

    void update(SDL_Rect &camera)
    {
        for(auto entity: this->getSystemEntities())
        {
            auto transform = entity.getComponent<TransformComponent>();

            if ((transform.position.x + (camera.w / 2.)) < Game::mapWidth)
            {
                camera.x = transform.position.x - (Game::windowsWidth / 2.);
            }
            if ((transform.position.y + (camera.h / 2.)) < Game::mapHeight)
            {
                camera.y = transform.position.y - (Game::windowsHeight / 2.);
            }
        }
        camera.x = (camera.x < 0) ? 0 : camera.x;
        camera.y = (camera.y < 0) ? 0 : camera.y;
        camera.x = (camera.x > camera.w) ? camera.w : camera.x;
        camera.y = (camera.y > camera.h) ? camera.h : camera.y;
    }
};

#endif /* GAMEENGINE_SOURCE_INCLUDE_SYSTEMS_CAMERAMOVEMENT_H */
